#version 330 core

// fragment shader
in vec2 UV;
out vec4 color;

uniform sampler2D myTextureSampler;


//////// fog insert 12apr18 ////////// (only for shark)

uniform int fogcolr=1; //  0=>noFog, 1=>white, 2=>brownish, 3=>purple, 4=>gray
uniform int foglevl=0; // 0=>noFog, 1=>light, 1=>Fog, 2=>heavy
const vec4 vFogColor=vec4(0.9,0.9,0.9,1.0); //normal white
const vec4 vDfogColor=vec4(0.7,0.7,0.7,1.0); //normal gray
const vec4 vSootColor=vec4(50.0/255,30.0/255,10.0/255,1.0); // brown
const vec4 vMystColor=vec4(0.7,0.5,0.7,1.0); // purple

const float fStart=0.0;
const float lEnd=20.0; // light fog
const float fEnd=12.0; // normal fog
const float hEnd= 8.0; // heavy fog

float getFogFactor(float rng)
{

	float fResult = (lEnd-rng)/(lEnd-fStart);
	if( foglevl>1 ) fResult = (fEnd-rng)/(fEnd-fStart);
	if( foglevl>2 ) fResult = (hEnd-rng)/(hEnd-fStart);
	
	fResult = 1.0-clamp(fResult, 0.0, 1.0);
	
	return sqrt(fResult);
}


//////// fog insert 12apr18 //////////



// halfspace discard uniforms: ////////////////////////////////////

uniform int iside=0; // 0=>noDiscard

uniform vec3 eyePos;

uniform vec3  port1;
uniform vec3  port2;

// halfspace discard uniforms end /////////////////////////////////

in vec4 aPos;


void main(){

	color = texture( myTextureSampler, UV).rgba;
	if( color.a < 0.1 ) discard;


	// addendum 12apr18:
	if(foglevl>0) {
		const float yc=-1.0;
		float dist2obj = length(aPos.xyz-eyePos);
		float dist = exp(-1.0*abs(aPos.y-yc)) * dist2obj;

		if( fogcolr==4 )
			color = mix(color,  vDfogColor, getFogFactor(dist)); //gray
		else if( fogcolr==3 )
			color = mix(color, vMystColor, getFogFactor(dist)); //purple
		else if( fogcolr==2 )
			color = mix(color, vSootColor, getFogFactor(dist)); //brown
		else if( fogcolr==1 )
			color = mix(color,  vFogColor, getFogFactor(dist)); //white

	}




	// fastrgv:  my halfspace-discard addendum:
	vec3 cP1 =  port1 - eyePos;
	vec3 cP2 =  port2 - eyePos;
	vec3 vP0 = aPos.xyz - eyePos;
	float PRAD = 0.80; // larger, but synched with adagate.adb
	float FOV1 = atan(PRAD/length(cP1));
	float FOV2 = atan(PRAD/length(cP2));
	vec3 nP1 = normalize(cP1);
	vec3 nP2 = normalize(cP2);
	vec3 nP0 = normalize(vP0);
	float d1=dot(nP0,nP1);
	float d2=dot(nP0,nP2);
	float a1=acos(d1);
	float a2=acos(d2);
	if( iside != 0 ) {
		if( (iside==2) && (a1>FOV1) ) discard;
		if( (iside==1) && (a2>FOV2) ) discard;
	}

}



//--
//-- Copyright (C) 2020  <fastrgv@gmail.com>
//--
//-- This program is free software: you can redistribute it and/or modify
//-- it under the terms of the GNU General Public License as published by
//-- the Free Software Foundation, either version 3 of the License, or
//-- (at your option) any later version.
//--
//-- This program is distributed in the hope that it will be useful,
//-- but WITHOUT ANY WARRANTY; without even the implied warranty of
//-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//-- GNU General Public License for more details.
//--
//-- You may read the full text of the GNU General Public License
//-- at <http://www.gnu.org/licenses/>.
//--

